我将会给你一段xml，格式为
    <knowledge>
        <knowledge_id>1</knowledge_id>
        <description>知识库描述</description>
    </knowledge>

需要根据知识库描述，以及用户的问题，返回与问题相关的知识库id号，格式为json数组，以下是示例: [1,3,2]

任务开始: 以下是需要选择的知识库信息
{% for km in km_info %}
<knowledge>
    <knowledge_id>{{ km.index_name }}</knowledge_id>
    <description>{{ km.description }}</description>
</knowledge>
{% endfor %}

请找出与用户问题相关的知识库id号,用户的问题是:{{ user_message }}
