Please come up with a question/answer pair, in the specified JSON format, for the following text:
----------------
{{text}}
----------------
generate {{size}} Q&A Pair

{% if extra_prompt %}
Additional instructions:
{{extra_prompt}}
{% endif %}